#!/usr/bin/env python
# coding: utf-8

from starlette.requests import Request

import ray
from ray import serve
import os
import numpy as np

from spherex_qc import SPHERExQCHandler as _SPHERExQCHandler

# download data
minio_endpoint = "http://210.110.233.67:9005"
bucket = "public"
file_path = "spherex-qc/zodicube2_4x6_21Mar.npy"
fn = os.path.join("data", os.path.basename(file_path))
url = f"{minio_endpoint}/{bucket}/{file_path}"
url, fn

if not os.path.exists(fn):
    print(f"downloading {url} -> {fn}")
    os.system(f"curl -o {fn} {url}")

print("file downloaded")

ray.init("auto")

# Load the data and put it on the shared memory
fn = "data/zodicube2_4x6_21Mar.npy"
data = np.load(fn)
zodi_data = dict((int(k), data[k]) for k in "123456")

r_zodi_data = ray.put(zodi_data)


class SPHERExQCHandler(_SPHERExQCHandler):
    "Derived class to suuport ray.get for data loading"
    def __init__(self, r_zodi_data):
        zodi_data = ray.get(r_zodi_data)
        _SPHERExQCHandler.__init__(self, zodi_data)


SPHERExQCDeployment = serve.deployment(SPHERExQCHandler)
SPHERExQCDeployment.deploy(r_zodi_data)


qc_handle = SPHERExQCDeployment.get_handle()


releases_info = {"releases": []}

RELEASENAME = "kasi20211021pre01"
RELEASEDATE = "20211012"
api_prefix = f"/api/release/{RELEASENAME}/query"

releases_info["releases"].append(dict(release_name="kasi20211021pre01",
                                      release_date="2021-10-21",
                                      api_prefix=api_prefix))


@serve.deployment(route_prefix=api_prefix)
class ReleaseHandler:
    def __init__(self, qc_handle):
        self.qc_handle = qc_handle

    async def __call__(self, request: Request):
        ra = request.query_params.get("ra", None)
        dec = request.query_params.get("dec", None)

        name = request.query_params.get("name", "")
        zodi = request.query_params.get("zodi", False)

        tbl = await qc_handle.query.remote(name, ra, dec, zodi=zodi)

        d = tbl.to_pandas().to_dict("split")
        return d


ReleaseHandler.deploy(qc_handle)


@serve.deployment(route_prefix=f"/api/releases")
class Releases:
    def __init__(self, releases_info):
        self.releases_info = releases_info

    def __call__(self, request: Request):
        return self.releases_info


Releases.deploy(releases_info)

